/*
 HMGGEN - Harbour Application Generator source code

 Copyright 2014 Frank Alleman <f.alleman@chello.nl>

 This program is free software; you can redistribute it and/or modify it under 
 the terms of the GNU General Public License as published by the Free Software 
 Foundation; either version 2 of the License, or (at your option) any later 
 version. 

 This program is distributed in the hope that it will be useful, but WITHOUT 
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS 
 FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

 You should have received a copy of the GNU General Public License along with 
 this software; see the file COPYING. If not, write to the Free Software 
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA (or 
 visit the web site http://www.gnu.org/).

 As a special exception, you have permission for additional uses of the text 
 contained in this release of HMG.

 The exception is that, if you link the HMG library with other 
 files to produce an executable, this does not by itself cause the resulting 
 executable to be covered by the GNU General Public License.
 Your use of that executable is in no way restricted on account of linking the 
 HMG library code into it.

 Parts of this project are based upon:

	HMG - Harbour Windows GUI library source code
	Copyright 2002-2010 Roberto Lopez <mail.box.hmg@gmail.com>
	http://sites.google.com/site/hmgweb/

	"Harbour GUI framework for Win32"
 	Copyright 2001 Alexander S.Kresin <alex@belacy.belgorod.su>
 	Copyright 2001 Antonio Linares <alinares@fivetech.com>
	www - http://www.harbour-project.org

	"Harbour Project"
	Copyright 1999-2008, http://www.harbour-project.org/

	"WHAT32"
	Copyright 2002 AJ Wos <andrwos@aust1.net> 

	"HWGUI"
  	Copyright 2001-2008 Alexander S.Kresin <alex@belacy.belgorod.su>

*/

#include <hmg.ch>
#include "hbclass.ch"

/*
public:
function WfmMakeCase(sDescrProcess, nRefId, sDossier, dDate, cTime)
function WfmUpdateStatus(sDescrProcess, sNameActivity, nCaseId, sStatus, sMessage, sContinue)
function OnCloseCase(nCaseId)
function WfmLookupCase(sDescrProcess, nObjectId)
function WfmPickupWorkitem(nCaseid, sNameActivity)// pick up workitem from outside 
function OnChangeStatus()
function WfmGetCurrentCaseId()
function WfmGetObjectId(nCaseId)
function WfmGetCurrentActivity()
function WfmLookupDossier(nCaseId)
function WfmUpdateDossier(nCaseId, cDossier)
function WfmLookupStatus(sNameActivity)
function WfmRecall(cNameProcess, cNameActivity, cStatusNew, bAction, nMaxRecalls, dRundate, lCount)
function WfmApplyChanges(cNameProcess, cNameActivity, bAction, dRundate, lCount)

private:
function HGW010_OnInit()
function HGW010_grid_OnInit(worklist)
function HGW010_OnDetails()
function HGW010_OnPickup()
function HGW010_OnForward()
function HGW010_OnRelease()
function HGW010_OnDelete()
function HGW010_OnMessage()

function HGW010_CloseActivity(nActiId)
function HGW010_NextActivityExists(cStatusActivity, activityDefinition)
function HGW010_StartNextActivity(sAbbrevActor, sDescrProcess, nCaseId, nObjectId, nActiId, sMessage, bNested)
function HGW010_GetNextActivities(cStatusActivity, oActivityDefinition)

function HGW020_OnApprove()
function HGW020_OnForward()

function HGW090_OnInit()
function HGW090_OnCancel()
function HGW090_OnApply()
*/

declare window HGW010
declare window HGW012
declare window HGW020
declare window HGW090
declare window HGW110

/*
	usage		: Shows refreshed inbox with workitems
	input		: 
	output	:
	last update	: 01-03-2014
*/
function HGW010_OnInit()
   local worklist := {}
   worklist := Activities():GetWorklist(worklist)
   HGW010_grid_OnInit(worklist)
return(nil)

function HGW010_grid_OnInit(worklist)
   local n, a, status
   HGW010.grid_1.DeleteAllItems
   for n := 1 to len(worklist)
      a := worklist[n, 1]
      if empty(a:abbrevActor)
         status := "Open"
      else
         if empty(a:dateEnd)
            status := "In process"
         else
            status := "Closed"
         endif
      endif
      HGW010.grid_1.AddItem({rtrim(worklist[n, 2]), str(worklist[n, 3]) , rtrim(worklist[n, 4]), ;
      a:abbrevActor, status, dtoc(a:dateStart), a:id})
   next
return(nil)

function HGW010_OnDetails()
   if isWindowDefined(HGW012)
      HGW012.Release
   endif   
   load window HGW012
   HGW012.Activate()
return(nil)

/*
	usage		: Assigns picked up workitem to current actor and starts function that handles activity
	input		: 
	output	:
	last update	: 01-03-2014
*/
function HGW010_OnPickup()
   local aItem, a, txdate := date(), txtime := time(), cb
   private s
   if HGW010.grid_1.Value = 0
      MsgInfo("First select a workitem from the list")
      return(nil)
   endif   
   // get id selected activity
   aItem := HGW010.grid_1.Item(HGW010.grid_1.Value)
   // get activity including its activityDefinition, process and case 		
   a := Activities():Get(aItem[7], Activity():New(), "activity") 
	if !empty(a:abbrevActor) .and. userprofile:userid != a:abbrevActor
		MsgInfo("Workitem already picked up by: " + a:abbrevActor)
	else
      a:abbrevActor := userprofile:userid
      /* don't update a:userid here */
      a:Save(txdate, txtime, a:userid, txdate)
		// set public variable identity
		nIdentity := a:case:idObject
		// perform actionblock
		s := "{||On" + Concatenate(a:activityDefinition:descrActionBlock) + "()}"
		cb := &s.
		eval(cb)
      aItem[4] := a:abbrevActor
      aItem[5] := a:descrStatus
      HGW010.grid_1.Item(HGW010.grid_1.Value) := aItem
	endif
return(NIL)

/*
	usage		: Forwards workitem to next activity in workflow process
	input		: 
	output	:
	last update	: 01-03-2014
*/
function HGW010_OnForward()
   local aItem, a, c, d, p, cb
   private s
   if HGW010.grid_1.Value = 0
      MsgInfo("First Select a workitem form the list")
   return(nil)
   endif   
   aItem := HGW010.grid_1.Item(HGW010.grid_1.Value)
   // get activity including its activityDefinition, process and case 		
   a := Activities():Get(aItem[7], Activity():New(), "activity") 
   c := a:case
   d := a:activityDefinition
   p := d:process
   // set public variable identity
   nIdentity := a:case:idObject
   if empty(a:abbrevActor)
      MsgInfo("First pick up an activity first")
   else
      if !empty(a:abbrevActor) .and. userprofile:userid != a:abbrevActor
         MsgInfo("Workitem in process by: " + abbrevActor)
      else
		 s := d:descrActionBlock
         if empty(s) 
            if HGW010_NextActivityExists(a:descrStatus, d)
               HGW010_CloseActivity(a:id)
               HGW010_StartNextActivity("", p:descrProcess, c:id, c:idObject, a:id, "")
            else
               MsgInfo("Workitem hasn't been picked up yet")
            endif
         else
            if d:indicAuto = "Y"
               s := "{|x, y|On" + s + "(x, y)}" 
               cb = &s.
               eval(cb, c:id, c:idObject)
            else   
               if d:indic4Eyes = "Y"
                  HGW020_OnApprove() 
               else
                  // defensive programming
               endif
            endif   
         endif
      endif
   endif
return(NIL)

/*
	usage		: Releases picked up workitem, so it can be picked up by someone else
   input		: 
	output	:
	last update	: 01-03-2014
*/
function HGW010_OnRelease()
   local aItem, a
   private s
   if HGW010.grid_1.Value = 0
      MsgInfo("First Select a workitem form the list")
   return(nil)
   endif   
   aItem := HGW010.grid_1.Item(HGW010.grid_1.Value)
   a := Activities():Get(aItem[7], Activity():New(), "activity") 
   a:abbrevActor := ""
   a:Save(date(), time(), a:userid, date())
   aItem[4] := a:abbrevActor
   HGW010.grid_1.Item(HGW010.grid_1.Value) := aItem
return(nil)

/*
	usage		: Deletes workitem and closes case
	input		: 
	output	:
	last update	: 01-03-2014
*/
function HGW010_OnDelete()
   local aItem, a, c
   private s
   if HGW010.grid_1.Value = 0
      MsgInfo("First Select a workitem form the list")
      return(nil)
   endif
   if MsgYesNo("Are you sure?", "Confirmation")
      aItem := HGW010.grid_1.Item(HGW010.grid_1.Value)
      a := Activities():Get(aItem[7], Activity():New(), "activity") 
      c := a:case
      a:abbrevActor := userprofile:userid
      a:descrStatus := "Deleted"
      a:Save(date(), time(), userprofile:userid, date())
      HGW010_CloseActivity(a:id)
      HGW010_StartNextActivity(userprofile:userid, "", c:id, c:idObject, a:id, "")     
      // OnCloseCase(c:id)
      HGW010.grid_1.DeleteItem(HGW010.grid_1.Value)
   endif   
return(nil)

/*
	usage		: Shows message that accompanies workitem to next activity
	input		: 
	output	:
	last update	: 01-03-2014
*/
function HGW010_OnMessage()
local aItem, a
   if HGW010.grid_1.Value = 0
      MsgInfo("First select a workitem form the list")
      return(nil)
   endif   
   aItem := HGW010.grid_1.Item(HGW010.grid_1.Value)
   // get activity including its activityDefinition, process and case 		
   a := Activities():Get(aItem[7], Activity():New(), "activity") 
   MsgInfo(a:descrMessage)
return(nil)

/*
	usage		: Closes current activity
	input		: 
	output	:
	last update	: 01-03-2014
*/
function HGW010_CloseActivity(nActiId)
   local a, txdate := date(), txtime := time()
   a := Activities():Get(nActiId, Activity():New(), "activity") 
   a:dateEnd := txdate
   a:timeEnd := txtime
   a:indicCurrent := "N"
   a:Save(txdate, txtime, userprofile:userid, txdate)
return(NIL)

/*
	usage		: makes a new workflow case 
	input		: description process
			  user's object id (e.g. customer id)
			  dossier (with xml structure for variables that must be passed during the process)		
			  startdate case
			  starttime case
	output	: case id
*/
function WfmMakeCase(sDescrProcess, nRefId, sDossier, dDate, cTime)
   local c, a, nProcId
	// goed testen want er wordt niet meer gecheckt of case al bestaat voor descrProcess en refid
   nProcId := Processes():GetProcessId(sDescrProcess)
   c := Case():New() 
   c:idProcess := nProcId
   if pcount() > 2
      c:descrDossier := sDossier
   endif
   c:idObject :=  nRefId
   c:descrStatus := "Opened"
   c:Save(dDate, cTime, userprofile:userid, dDate)
   a := Activity():New()
   a:case:id := c:id
   a:activityDefinition:id := ActivityDefinitions():GetIdFirstActivity(nProcId)
   a:dateStart := dDate
   a:timeStart := cTime
   a:descrStatus := "Case opened"
   a:abbrevActor := userprofile:userid
   // close activity immediately
   a:dateEnd := dDate
   a:timeEnd := cTime
   a:indicCurrent := "N"
   a:Save(dDate, cTime, userprofile:userid, dDate)
   HGW010_StartNextActivity(userprofile:userid, sDescrProcess, c:id, c:idObject, a:id, "")
return(c:id)

/*
	usage		: updates the state of an activity
	input		: description process
			  name activity
			  case id
			  new status activity		
			  message that must be passes to next activity
			  text "continue" (optional).W
			  When "continue" has been specified, the current activity is closed and
			  the next activity is started and, if indication automatic has been set, performed 
			  as well.			 
	output	: -
*/
function WfmUpdateStatus(sDescrProcess, sNameActivity, nCaseId, sStatus, sMessage, sContinue)
   local a, c, txdate := date(), txtime := time()
   a := Cases():GetCurrentActivity(nCaseId, sNameActivity)
   /* defensive programming */
   if a:id = 0
      /* probably no next activity defined for status "Case opened" */
      MsgInfo("Activity not found, please contact your servicedesk (error ACTI1)")
   else
      c := a:case
      a:descrStatus := sStatus
      a:descrMessage := sMessage
      a:Save(txdate, txtime, userprofile:userid, txdate)   
      if pcount() > 5
         HGW010_CloseActivity(a:id)
         HGW010_StartNextActivity("SYS", sDescrProcess, c:id, c:idObject, a:id, "") 
      endif
   endif   
return(NIL)

/*
	usage		: Checks whether activity has follow up activities or not
	input		:  
	output	:
	last update	: 01-03-2014
*/
function HGW010_NextActivityExists(cStatusActivity, activityDefinition)
   local retval := .T., resultset := {}
   resultset := HGW010_GetNextActivities(cStatusActivity, activityDefinition)
   if empty(resultset)
      retval := .F.
   endif
return(retval)

/*
	usage		: Starts next activity/activities in workflow process
	input		: 
	output	:
	last update	: 01-03-2014
*/
function HGW010_StartNextActivity(sAbbrevActor, sDescrProcess, nCaseId, nObjectId, nActiId, sMessage, bNested)
   local resultset, a, aNew, d, idNa, n, nRefCaseId, nRefObjectId, nRefActiId, cb, txdate := date(), txtime := time()
   private s
   a := Activities():Get(nActiId, Activity():New(), "activity") 
   /* defensive programming */
   if a:id = 0
      MsgInfo("Activity not found, please contact your servicedesk (error ACTI2)")
      return(nil)
   endif 
   d := a:activityDefinition
   if d:nameActivity == "Open case"
      txdate := a:txdate
      txtime := a:txtime
   endif   
   resultset := HGW010_GetNextActivities(a:descrStatus, d)
   for n := 1 to len(resultset)
	   idNa := resultset[n]
      // get activity definition of next activity 
      dna := activityDefinitions():Get(idNa, ActivityDefinition():New(), "nextActivity")
      /* defensive programming */
      if dna:id = 0
         MsgInfo("Activity definition not found, please contact your servicedesk (error DACT1)")
         exit
      endif   
      // note that case is still the same
      // make new activity
      aNew := Activity():New()
      aNew:case:id := a:case:id
      aNew:activityDefinition:id := dna:id
      aNew:dateStart := txdate
      aNew:timeStart := txtime
      aNew:indicCurrent := "Y"
      aNew:numberOfMonthsRecallterm := dna:numberOfMonthsRecallterm
      aNew:descrStatus := dna:descrInitialStatus 
      aNew:descrMessage := sMessage
      if dna:indicAuto == "Y"  
         aNew:abbrevActor := "SYS"
      else
         if sAbbrevActor == "SYS"
            aNew:abbrevActor := ""
         else
            aNew:abbrevActor := sAbbrevActor
         endif
      endif
      aNew:Save(txdate, txtime, userprofile:userid, txdate)
      // save keys
      nRefCaseId := aNew:case:id
      nRefObjectId := aNew:case:idObject
      nRefActiId := aNew:id               
      // start auto activity directly
      if dna:indicAuto == "Y"
         // start use case
         s := "{|x, y|On" + Concatenate(dna:descrActionBlock) + "(x, y)}"
         cb := &s.
         if eval(cb, aNew:case:id, aNew:case:idObject)
            HGW010_CloseActivity(nRefActiId)
            /* note that now bNested is .T. */
            HGW010_StartNextActivity("SYS", sDescrProcess, nRefCaseId, nRefObjectId, nRefActiId, "", .T.)
         endif   
      endif
   next
   if IsWindowDefined(HGW010) .and. pcount() = 6
      // HGW010_grid_OnInit()
      HGW010_OnInit()
   endif   
return(nil)

/*
	usage		: Retrieves array with id's of folluw up activities
	input		: 
	output	:
	last update	: 01-03-2014
*/
function HGW010_GetNextActivities(cStatusActivity, oActivityDefinition)
   local aTemp, resultset := {}, n
   if cStatusActivity == "Deleted"
      aTemp := ActivityDefinitions():Find(oActivityDefinition:process:id, {})
      for n := 1 to len(aTemp)
         if aTemp[n]:nameActivity == "Close case"
            aadd(resultset, aTemp[n]:id)
            exit
         endif
      next
   else
      resultset := oActivityDefinition:SelectNextActivities(cStatusActivity, {})
   endif   
return(resultset)

/*
	usage		: activates use case "Close case" to close the case
	input		: case id
	output	:
	last update	: 01-03-2014
*/
function OnCloseCase(nCaseId)
   local a, c, d, n, txdate := date(), txtime := time()
   c := Cases():Get(nCaseId, Case():New(), "case")
   c:descrStatus := "Closed"
   c:Save(txdate, txtime, userprofile:userid, txdate)
   for n := 1 to len(c:activities)
      a := c:activities[n]
      d := ActivityDefinitions():Get(a:activityDefinition:id, ActivityDefinition():New(), "case")
      if d:nameActivity == "Close case"
         a:descrStatus := "Case closed"
         a:Save(txdate, txtime, userprofile:userid, txdate)
         exit
      endif
   next
return(.T.)

/*
	usage		: retuns id of current case for given process and object id 
	input		: description process
			  user's object id (e.g. customer id)
	output	: case id
	last update	: 01-03-2014	
*/
function WfmLookupCase(sDescrProcess, nObjectId)
   /* Beware that this function returns the caseid of the first (oldest) case encountered */
   local cases, retval := 0, c, n 
   cases := Cases():Find(Processes():GetProcessId(sDescrProcess), {})
   for n := 1 to len(cases)
      c := cases[n]
      if c:descrStatus == "Opened" .and. c:idObject = nObjectId
         retval := c:id
         exit
      endif     
   next
return(retval)

/*
	usage		: returns status other activity within same case
	input		: name activity
	output	: status activity
	last update	: 01-03-2014
*/
function WfmLookupStatus(sNameActivity)
   local aItem, a, c, n, status := "Unknown"
   aItem := HGW010.grid_1.Item(HGW010.grid_1.Value)
   // get activity including its activityDefinition, process and case 		
   a := Activities():Get(aItem[7], Activity():New(), "activity") 
   // get ALL activities
   c := Cases():get(a:case:id, Case():New(), "case")
   for n := 1 to len(c:activities)
      if c:activities[n]:activityDefinition:nameActivity == sNameActivity
         status := c:activities[n]:descrStatus
         exit
      endif   
   next
return(status)

/*
	usage		: assigns workitem (activity) to current user 
	input		: case id
			  name activity
	output	: -
	last update	: 01-03-2014	
*/
function WfmPickupWorkitem(nCaseid, sNameActivity)
local a, c, d, n, txdate := date(), txtime := time()
   c := Cases():Get(nCaseId, Case():New(), "case")
   for n := 1 to len(c:activities)
      a := c:activities[n]
      d := a:activityDefinition
      if d:nameActivity == "sNameActivity" .and. a:indicCurrent == "Y"
         a:abbrevActor := userprofile:userid
         a:Save(txdate, txtime, userprofile:userid, txdate)
         exit
      endif
   next
return(nil)

/*
	usage		: workitem predecessors 
	input		: -
	output	: -
	last update	: 01-03-2014	
*/
function HGW012_OnInit()
   local aItem, n, c, a
   if HGW010.grid_1.Value = 0
      MsgInfo("First select a workitem form the list")
      return(nil)
   endif   
   aItem := HGW010.grid_1.Item(HGW010.grid_1.Value)
   c := cases():Get(val(aItem[2]), Case():New(), "case") 
   for n := 1 to len(c:activities)
      a := c:activities[n]
      HGW012.grid_1.AddItem({a:activityDefinition:process:nameProcess, str(c:id), a:activityDefinition:nameActivity, ;
      a:abbrevActor, a:descrStatus, dtoc(a:dateStart)+" "+a:timeStart, dtoc(a:dateEnd)+" "+a:timeEnd, a:id})
   next
return(nil)

function HGW012_OnClose()
   HGW012.Release
return(nil)

/*
	usage		: activates approve form 
	input		: -
	output	: -
	last update	: 01-03-2014	
*/
function HGW020_OnApprove()
   if isWindowDefined(HGW020)
      HGW020.Release
   endif   
   load window HGW020
   HGW020.Activate()
return(nil)

function HGW020_OnInit()
return(nil)

function HGW020_OnForward()
	local aItem, a, c, d, p, txdate := date(), txtime := time(), cUserId := ""
	aItem := HGW010.grid_1.Item(HGW010.grid_1.Value)
   // get activity including its activityDefinition, process and case 		
   a := Activities():Get(aItem[7], Activity():New(), "activity") 
	c := a:case
	d := a:activityDefinition
	p := d:process
	if HGW020.AKKOORD.Value = .T. 
		a:descrStatus := "Approved"
		a:descrMessage := HGW020.REDEN.Value 
	else
      cUserId := a:userid   
		a:descrStatus := "Rejected"
		a:descrMessage := HGW020.REDEN.Value 
	endif
	a:Save(txdate, txtime, userprofile:userid, txdate)
	HGW010_CloseActivity(a:id)
	HGW010_StartNextActivity(cUserId, p:nameProcess, c:id, c:idObject, a:id, a:descrMessage)
	HGW020.Release
return(nil)

/*
	usage		: activates use case "Change status" to change status of activity
			  e.g. state transition to "Response received" for activity "Wait for response" 			         		 
	input		:
	output	:
	last update	: 01-03-2014
*/
function OnChangeStatus()
   if isWindowDefined(HGW090)
      HGW090.Release
   endif   
   load window HGW090
   HGW090.Activate()
return(nil)

function HGW090_OnInit()
  	// get states from nextactivities
	local aItem, a, d, na, txdate := date(), txtime := time(), n
	aItem := HGW010.grid_1.Item(HGW010.grid_1.Value)
  	// get activity including its activityDefinition, process and case 		
  	a := Activities():Get(aItem[7], Activity():New(), "activity") 
	d := a:activityDefinition
  	HGW090.STATUS.DeleteAllItems()
  	HGW090.STATUS.AddItem(a:descrStatus)
	for n := 1 to len(d:nextActivities)
		na := d:nextActivities[n]
     	HGW090.STATUS.AddItem(na:descrStatus)
   next
	HGW090.STATUS.Value := 1
return(nil)

function HGW090_OnCancel()
   HGW090_OnInit()
return(nil)

function HGW090_OnApply()
	local aItem, a, c, d, p, txdate := date(), txtime := time(), status 
	aItem := HGW010.grid_1.Item(HGW010.grid_1.Value)
   // get activity including its activityDefinition, process and case 
   a := Activities():Get(aItem[7], Activity():New(), "activity") 
	c := a:case
	d := a:activityDefinition
	p := d:process
   status := HGW090.STATUS.Item(HGW090.STATUS.Value)
   WfmUpdateStatus(p:nameProcess, d:nameActivity, c:id, status, "", "continue")
	HGW090.Release
return(nil)

/*
	usage		: returns case id of current selected activity in inbox  
	input		: -
	output	: case id
	last update	: 01-03-2014	
*/
function WfmGetCurrentCaseId()
	local aItem, retval := 0 
   if HGW010.grid_1.Value > 0
      aItem := HGW010.grid_1.Item(HGW010.grid_1.Value)
      retval := val(aItem[2])
   endif
return(retval)   

/*
	usage		: returns objectid of given case
	input		: case id
	output	: object id
	last update	: 01-04-2014	
*/
function WfmGetObjectId(nCaseId)
   local case, retval := "" 
   case := Cases():Get(nCaseId, Case():New(), "activity")
   retval := case:idObject
return(retval)

/*
	usage		: returns name activity of current selected activity in inbox  
	input		: -
	output	: case id
	last update	: 01-03-2014	
*/
function WfmGetCurrentActivity()
	local aItem, retval := "" 
   if HGW010.grid_1.Value > 0
      aItem := HGW010.grid_1.Item(HGW010.grid_1.Value)
      retval := aItem[3]
   endif
return(retval)   
    
/*
	usage		: returns dossier of given case  
	input		: case id
	output	: dossier (with xml message)
	last update	: 01-03-2014	
*/
function WfmLookupDossier(nCaseId)
   local case, retval := "" 
   case := Cases():Get(nCaseId, Case():New(), "activity")
   retval := case:descrDossier
return(retval)

/*
	usage		: updates dossier of case  
	input		: case id
			: dossier (with xml message)
	output	: -
	last update	: 01-03-2014	
*/
function WfmUpdateDossier(nCaseId, cDossier)
   local case
   case := Cases():Get(nCaseId, Case():New(), "activity")
   case:descrDossier := cDossier
   case:Save(date(), time(), userprofile:userid, date())
return(nil)

/*
	usage		: recall function 
	input		: name process
              name activity
              endstatus when max recalls have exceeded 
              codeblock for user defined action, format {|x,y,z|...}  
              max recalls, must be a multiple of the recallterm, e.g. 3 for 3 recall rounds
              rundate
              count indication
	output	: -
	last update	: 23-04-2014	
*/
function WfmRecall(cNameProcess, cNameActivity, cStatusNew, bAction, nMaxRecalls, dRundate, lCount)
   local aResultset := {}, n, oActivity, txtime := time(), nCaseId, nObjectId, nRecallterm, nCount := 0
   aResultset := Activities():QueryOnRappeltermExpired(cNameProcess, cNameActivity, aResultset, dRundate)
   for n := 1 to len(aResultset)
      if pcount() > 6 .and. lCount
         nCount++
      else 
         Barchart()
         oActivity   := aResultset[n, 1]
         nCaseId     := aResultset[n, 2]
         nObjectId   := aResultset[n, 3]
         nRecallterm := aResultset[n, 4]
         if oActivity:numberOfMonthsRecallterm <= (nRecallterm * nMaxRecalls)
            /* send specific recall letter */
            eval(bAction, oActivity, nCaseId, nObjectId)
            /* recall after another term */ 
            oActivity:numberOfMonthsRecallterm += nRecallterm
            oActivity:Save(dRundate, txtime, "SYS", dRundate)   
         else
            WfmUpdateStatus(cNameProcess, cNameActivity, nCaseId, cStatusNew, "", "continue")
         endif
      endif   
   next
return(nCount)

/*
	usage		: applies changes for given activity on startdate
	input		: name process
              name activity
              codeblock for user defined action, format {|x,y,z|...}  
              rundate
              count indication (optional) 
   output	: -
	last update	: 23-04-2014	
*/
function WfmApplyChanges(cNameProcess, cNameActivity, bAction, dRundate, lCount)
   local aResultset := {}, n, oActivity, nCaseId, nCount := 0
   aResultset := Activities():QueryOnOpenActivities(cNameProcess, cNameActivity, aResultset)
   for n := 1 to len(aResultset)
      if pcount() > 4 .and. lCount
         nCount++
      else 
         Barchart()
         oActivity   := aResultset[n, 1]
         nCaseId     := aResultset[n, 2]
         eval(bAction, oActivity, nCaseId, dRundate)
      endif   
   next
return(nCount)

function Barchart(reccount)
static counter
if pcount() = 1
   counter := 0
   HGW110.ProgressBar_1.RangeMax := reccount
   HGW110.ProgressBar_1.Value := 0
else
   counter++
   if counter <= HGW110.ProgressBar_1.RangeMax
      HGW110.ProgressBar_1.Value := counter
   endif   
endif
return(nil)

**** end of file ****



